///////////////////////////////////////////////////////////////////////////////////////
// HEPTopTagger example - (M. Takeuchi)    //////////////////
// Example code with Top-Tagger - according to arXiv:1006.2833 (T.Plehn, M.Spannowsky, M.Takeuchi, D.Zerwas)    //////////////////
// Code last modified on Oct 6th, 2016                                ///////////////////
// by Michihisa Takeuchi      michihisa.takeuchi@ipmu.jp                     ///////////////////
///////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <sstream> 
#include <iomanip>  
#include <cmath>
#include <iostream>
#include <fstream>
#include <math.h>
#include <string.h>
#include <exception>
#include <cstdlib>
#include <cstdio>
#include <algorithm>
//#include <ext/hash_map>
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"
#include "fastjet/CDFJetCluPlugin.hh"
#include "fastjet/internal/BasicRandom.hh"
using namespace std;

//////////////////
#include <fastjet/tools/MassDropTagger.hh>  // needed for HEPTopTagger
#include <fastjet/tools/Filter.hh>   // needed for HEPTopTagger
#include <fastjet/tools/Pruner.hh>   // needed for HEPTopTagger
#include <fastjet/tools/HEPTopTagger.hh>
//////////////////
using namespace fastjet;
#include "example.hh"
#include "output_gnuplot_display.hh"

//const PseudoJet Zero_PseudoJet(0.,0.,0.,0.);

int main(int argc, char *argv[])
{
  double topmass=172.3;
  double wmass=80.4;
  vector<fastjet::PseudoJet> hadrons;
  ifstream fin("sample_event.dat",ifstream::in );  
  ReadEvent(fin,hadrons); 
  ofstream fout("sample_event_display.dat",ifstream::out );   
  ofstream fout2("sample_event_out.dat",ifstream::out );  
  output_vec_pseudojet(fout2,hadrons);

  // start of granularization of the hadronic calorimeter to redefine hadrons
  double eta_cell=0.1;
  double phi_cell=0.1;
  double pt_cutoff=0.5;
  vector<fastjet::PseudoJet> gran_hadrons 
    = gran_jets(hadrons,eta_cell,phi_cell,pt_cutoff);
  
  // jet definition
  double conesize=1.5;
  fastjet::JetDefinition jet_def(fastjet::cambridge_algorithm,conesize);
  // run the jet finding; find the hardest jet
  fastjet::ClusterSequence clust_seq(gran_hadrons, jet_def);  
  double ptmin_jet=150.;
  vector<fastjet::PseudoJet> jets = sorted_by_pt(clust_seq.inclusive_jets(ptmin_jet));
 
  cout << "#----- HEPTopTagger definition -----#" << endl;
  fastjet::HEPTopTagger heptop1(50.);
  //heptop1.check_massratio_first(true);
  //heptop1.set_top_range(150.,200.);
  //heptop1.set_mass_ratio_cut(0.35,0.2,1.3);
  //heptop1.set_max_subjet_mass(50.);
  //heptop1.set_Rprun(R);
  //heptop1.set_filtering_jetalgorithm(kt_algorithm);
  //heptop1.set_reclustering_jetalgorithm(kt_algorithm);
  //heptop1.return_only_tagged(true);
  cout << heptop1.description() <<endl;
  cout << heptop1.description_filter() <<endl;
  cout << heptop1.description_prune() <<endl;
  cout << "#----- HEPTopTagger definition -----#" << endl;

  for(unsigned ijet=0; ijet<jets.size(); ijet++){
    cout << "--------- checking jet["<< ijet << "] ----------" << endl;
    PseudoJet taggedtop=heptop1(jets[ijet]);
    if(taggedtop == PseudoJet()){
      cout << "jet["<< ijet << "] is not a top-jet" << endl;
    }
    if(taggedtop != PseudoJet()) // heptop1.is_maybe_top();
      {
	cout << "####### top cand found #####" << endl;
	bool istop_bool = taggedtop.structure_of<HEPTopTagger>().is_tagged();
	if(istop_bool) cout << "=== this top cand is top tagged ! === "<< istop_bool << endl;
	if(!istop_bool) cout << "*** this top cand is not tagged ! *** "<< istop_bool << endl;
	cout << "Jet_orig   : " 
	     << "n: "<< jets[ijet].constituents().size() 
	     << ", M: "<< jets[ijet].m() 
	     << ", pt: "<< jets[ijet].pt() << endl;

	double prunedmass = taggedtop.structure_of<HEPTopTagger>().pruned().m();
	double unfilteredmass = taggedtop.structure_of<HEPTopTagger>().unfiltered().m();
	double cos = taggedtop.structure_of<HEPTopTagger>().cos_theta_W();	
	double delta_top= taggedtop.structure_of<HEPTopTagger>().delta_top();
	cout << "pruned mass: " <<  prunedmass << endl;
	cout << "unfiltered mass: " <<  unfilteredmass << endl;
	cout  << "cos:" << cos << endl;
	cout  << "delta_top:" << delta_top << endl;

	cout << "---------- constitutes ----------" << endl;
	vector<PseudoJet> top_consti=taggedtop.constituents();
	cout << "constituents n: "<< top_consti.size() << endl;
	cout << "W consti= n:" << taggedtop.structure_of<HEPTopTagger>().W().constituents().size() 
	     << ", pt: " << taggedtop.structure_of<HEPTopTagger>().W().pt() 
	     << ", M: " << taggedtop.structure_of<HEPTopTagger>().W().m() << endl;;
	cout << "b consti= n:" << taggedtop.structure_of<HEPTopTagger>().non_W().constituents().size() 
	     << ", pt: " << taggedtop.structure_of<HEPTopTagger>().non_W().pt() 
	     << ", M: " << taggedtop.structure_of<HEPTopTagger>().non_W().m()<< endl;
	cout << "W1 consti= n:" << taggedtop.structure_of<HEPTopTagger>().W().pieces().at(0).constituents().size() 
	       << ", pt: " << taggedtop.structure_of<HEPTopTagger>().W().pieces().at(0).pt() 
	     << ", M: " << taggedtop.structure_of<HEPTopTagger>().W().pieces().at(0).m()<< endl;
	cout << "W2 consti= n:" << taggedtop.structure_of<HEPTopTagger>().W().pieces().at(1).constituents().size() 
	     << ", pt: " << taggedtop.structure_of<HEPTopTagger>().W().pieces().at(1).pt() 
	     << ", M: " << taggedtop.structure_of<HEPTopTagger>().W().pieces().at(1).m()<< endl;
	
	cout << "------ regarded structure -------" << endl;	
	PseudoJet rejected_MD=join(taggedtop.structure_of<HEPTopTagger>().rejected_by_massdrop());
	PseudoJet rejected_F=join(taggedtop.structure_of<HEPTopTagger>().rejected_by_filter());
	PseudoJet not_selected=join(taggedtop.structure_of<HEPTopTagger>().not_selected());
	PseudoJet combined=join(taggedtop,rejected_MD,rejected_F,not_selected);		     	
	cout << "tagged   : " 
	     << "n: "<< taggedtop.constituents().size() 
	     << ", M: "<< taggedtop.m() 
	     << ", pt: "<< taggedtop.pt() << endl;
	cout << "Rejected_MD: " 
	     << "n: "<< rejected_MD.constituents().size() 
	     << ", M: "<< rejected_MD.m() 
	     << ", pt: "<< rejected_MD.pt() << endl;	      
	cout << "Rejected_F : " 
	     << "n: "<< rejected_F.constituents().size() 
	     << ", M: "<< rejected_F.m() 
	     << ", pt: "<< rejected_F.pt() << endl;
	cout << "not_sel    : " 
	     << "n: "<< not_selected.constituents().size() 
	     << ", M: "<< not_selected.m() 
	     << ", pt: "<< not_selected.pt() << endl;	  
	cout << "Combined   : " 
	     << "n: "<< combined.constituents().size() 
	     << ", M: "<< combined.m() 
	     << ", pt: "<< combined.pt() << endl;

	// tagged top to gnuplot display
	if(istop_bool){
	  PseudoJet top=taggedtop;
	  PseudoJet b=taggedtop.structure_of<HEPTopTagger>().non_W();
	  PseudoJet W1=taggedtop.structure_of<HEPTopTagger>().W().pieces().at(0);
	  PseudoJet W2=taggedtop.structure_of<HEPTopTagger>().W().pieces().at(1);
	  cout << "top mass: " << top.m() << endl;
	  cout << "bottom mass: "<< b.m() << endl;
	  cout << "W mass: "<< (W1+W2).m() << endl;
	  // for event display
	  set_header_for_display(fout);
	  circle(fout,top,0.1,1);
	  circle(fout,b,0.1,2);
	  circle(fout,W1,0.1,3);
	  circle(fout,W2,0.1,3);
	  set_footer_for_display(fout);
	}
      }
    }
  return 0;  // end of main  
}
